const async = require('async');
const pg = require('pg').Client;
const https = require('https');
const AWS = require('aws-sdk');

const db_config = {
    region: process.env.AWS_REGION,
    httpOptions: {
        agent: new https.Agent({
            ciphers: 'ALL',
            secureProtocol: 'TLSv1_method'
        })
    }
};

const dynamoDB = require('aws-dynamodb')(db_config);
const table = 'fndy_object_type_' + process.env.ENV;
const limit = 500;
const TAG = 'SQL TRIGGER ADVANCED SEARCH >';

exports.handler = function(event, context, callback) {

    console.log(`${TAG} EVENT: ${JSON.stringify(event)}`);

    let env = 'dev';

    if(typeof process.env.ENV == 'string'){
        env = process.env.ENV.toLowerCase();
    }

    const region = process.env.AWS_REGION || "eu-central-1";
    const rdsPass = "RDS_PASS_" + env;
    let ssm = new AWS.SSM({region:region});

    let pgClient = new pg();

    function init(callbackInit) {
        console.log(`${TAG} INIT`);

        var params = {
            Name: rdsPass, /* required */
            WithDecryption: true
        };
        ssm.getParameter(params, function(err, data) {
            if (err) {
                console.log("error",err, err.stack);
                return callbackInit(err);
            } // an error occurred
            else    {
                console.log("data",data);           // successful response
                pgClient = new pg({
                    host: process.env.DB_HOST,//'fndy-search-result.cildgunnxgsj.us-west-2.rds.amazonaws.com',
                    port: 5432,
                    user: 'search_user',
                    password: data.Parameter.Value,
                    database: 'AdvancedSearch'
                });
                pgClient.connect(function(err) {
                    if (err) {
                        console.error('connection error', err.stack);
                        return callbackInit('PG connect error');
                    } else {
                        callbackInit(null);
                    }
                });
            }
        });
    }

    function getDeviceTypes(cbGetDeviceTypesFull){
        function GetLastKeys(cbGetLastKeys) {
            var last_keys = [null];

            (function recursive_call($lastKey) {

                dynamoDB
                    .table(table)
                    .select('COUNT')
                    .resume($lastKey)
                    .limit(limit)
                    .scan(function (err) {

                        if (err) {
                            return cbGetLastKeys(err);
                        }

                        if (this.LastEvaluatedKey === null) {
                            console.log(last_keys);
                            return cbGetLastKeys(null, last_keys);
                        } else {
                            var $this = this;
                            setTimeout(function () {
                                last_keys.push($this.LastEvaluatedKey);
                                console.log(last_keys);
                                recursive_call($this.LastEvaluatedKey);
                            }, 0);
                        }

                    });

            })(null);

        }

        function GetDeviceTypes(last_keys, cbGetDeviceTypes) {
            async.map(last_keys, function (last_key, cb) {

                dynamoDB
                    .table(table)
                    .resume(last_key)
                    .limit(limit)
                    .scan(function (err, data) {

                        if (err) {
                            return cb(err);
                        }
                        cb(null, data);

                    });

            }, function (err, result) {

                if (err) {
                    return cbGetDeviceTypes(err);
                }
                return cbGetDeviceTypes(null, [].concat.apply([], result));

            });

        }
        console.log(cbGetDeviceTypesFull);
        async.waterfall([
            GetLastKeys,
            GetDeviceTypes
        ], function (err, result) {
            console.log("finsihed");
            if (err) {
                return cbGetDeviceTypesFull(err);
            }
            cbGetDeviceTypesFull(null, result);
        });
    }

    function createTables(types, cbCreateTables){
        async.map(types, (type, cbMap)=>{
            let mandatoryFields = {
                deviceId: 'VARCHAR(100)',//'String',
                accountId: 'VARCHAR(100)',//'String',
                typeId: 'VARCHAR(100)',//'String',
                epochTime: 'BIGINT'//'Integer'
            };

            let fieldList= '';

            for (let fieldName in mandatoryFields){

                fieldList += `"${fieldName}" ${mandatoryFields[fieldName]}, `;
            }

            for (let fieldName in type){
                let typeField = type[fieldName];
                if(typeField && typeField.characteristics && Array.isArray(typeField.characteristics)) {

                    fieldList += `"${fieldName}" ${selectTypeForColumn(type[fieldName]['data_type'])}, `;
                }
            }

            let requestString = `CREATE TABLE IF NOT EXISTS "_history_type_${type.typeId}" (${fieldList.slice(0, -2)})`;

            pgClient.query(requestString, cbMap)
        }, cbCreateTables)
    }

    function selectTypeForColumn(typeFromDynamoDB){

        switch (typeFromDynamoDB) {
            case "Integer":
                return 'BIGINT';
            case "Decimal":
                return 'NUMERIC';
            default:
                return 'TEXT';
        }
    }

    async.waterfall([
        init,
        getDeviceTypes,
        createTables
    ], function (err, result) {

        pgClient.end();

        if (err) {
            console.error('error:', err);
            const response = {
                status: 400,
                error: err.message ? err.message : err
            };
            return context.succeed(JSON.stringify(response));
        }

        return context.succeed(result);

    });
};
